local ShowDebugMessages = Game.IsTutorialLogging();

COMPLETE  = 0; -- Successfully completed this task
DISMISSED = 1; -- Tutorial display was seen and dismissed by player
INVALID   = 2; -- Missed the boat on this task
ACTIVE    = 3; -- This task can be evaluated
INACTIVE  = 4; -- This task cannot currently be evaluated

ADVISOR_SCIENCE  = AdvisorTypes.ADVISOR_SCIENCE;
ADVISOR_ECONOMIC = AdvisorTypes.ADVISOR_ECONOMIC;
ADVISOR_FOREIGN  = AdvisorTypes.ADVISOR_FOREIGN;
ADVISOR_MILITARY = AdvisorTypes.ADVISOR_MILITARY;

-- this is our global info table that will be used in various tutorials
GlobalTutorialInfo = {};

function Tutorial(arg)
	arg.Index = table.count(GlobalTutorialInfo);
	GlobalTutorialInfo[arg.ID] = arg;
end

-- serialized data?
local EverHealedUnit = false;
local bEverCityBombard = false;
local iMaxThingsCityBuilt = 0;
local NumHappinessResourcesConnected = 0;
local strID;

------------------------------------------------------------
------------------------------------------------------------
-- debug functions
function dprint(msg)
	if (ShowDebugMessages) then
		print(msg);
	end
end

------------------------------------------------------------
------------------------------------------------------------
-- utility functions
function GetPlayer ()
	local iPlayerID = Game.GetActivePlayer();
	if (iPlayerID < 0) then
		print("Error - player index not correct");
		return nil;
	end

	if (not Players[iPlayerID]:IsHuman()) then
		print("Error - player is not human");
		return nil;
	end;

	return Players[iPlayerID];
end

function IsStrictlyWorker (pUnit)
	if (pUnit:WorkRate() > 0 and not pUnit:IsCombatUnit() and pUnit:GetDomainType() == DomainTypes.DOMAIN_LAND and pUnit:GetSpecialUnitType() == -1) then
		return true;
	else
		return false;
	end	
end

------------------------------------------------------------
------------------------------------------------------------
function LostAllMilitaryUnitsCheck ()
	local player = GetPlayer();
	if (player == nil) then
		dprint("Error - player index not correct");		
		return INACTIVE;	
	end
	
	if (Game.GetGameTurn() > 100) then
		return COMPLETE;
	end

	if (Game.GetGameTurn() < 25) then
		return INACTIVE;
	end
	
	--if (player:GetNumMilitaryUnits() > 0) then
	--	return INACTIVE;
	--end
	
	-- if there is a unit in friendly territory, we're good for now
	for v in player:Units() do
		local pPlot = v:GetPlot();
		if (v:IsCombatUnit() and pPlot:GetOwner() == v:GetOwner()) then
			return INACTIVE;
		end
	end
	
	-- see if any city has a military unit queued next
	for v in player:Cities() do
		local UnitType = v:GetProductionUnit();
		if (UnitType >= 0 and GameInfo.Units[UnitType].Combat > 0) then
			dprint("City has military unit queued");
			return INACTIVE;
		end
	end
	
	-- see if any cities are done with production
	for v in player:Cities() do
		if (v:GetProduction() == 0) then
			return ACTIVE;	
		end
	end
			
	return INACTIVE;
end

Tutorial { 
	ID = "LOST_ALL_MILITARY",         
	ActivateButtonText = false, 
	Advisor = ADVISOR_MILITARY, 
	CheckFunction = LostAllMilitaryUnitsCheck,	    
	UnitIndexFunction = nil,					    
	PlotFunction = nil,				      
	MinTurn =  1, 
	MinTurnTime = -1, 
	TurnsCheck =  0, 
	PesterAgainTurn =  -1,
	TutorialLevel = 0,
	Concept1 = "CONCEPT_CITY_CONSTRUCTING_UNITS",
	Concept2 = "CONCEPT_UNITS_COMBAT_UNITS",
	Concept3 = "CONCEPT_VICTORY_DEFEAT",	
};

------------------------------------------------------------
------------------------------------------------------------
function GoodyHutCheck ()
	local player = GetPlayer();
	if (player == nil) then
		dprint("Error - player index not correct");		
		return -1;	
	end
	
	if (player:GetEverPoppedGoody()) then
		--dprint("Player has popped goody");
		return COMPLETE;
	end	

	-- if there is a unit in route, mark the step as inactive for now?
	if (player:GetAnyUnitHasOrderToGoody()) then
		--dprint("Player has a unit directed to get the goody");
		return INACTIVE;
	end
  
	if (player:IsAnyGoodyPlotAccessible()) then
		return ACTIVE;
	end
  
	return INACTIVE;
end

function GetGoodyHutPlot ()

	local player = GetPlayer();
	if (player == nil) then
		dprint("Error - player index not correct");		
		return nil;	
	end

	return player:GetClosestGoodyPlot();	
end

Tutorial { 
	ID = "GO_TO_GOODY_HUT",           
	ActivateButtonText = true,	
	Advisor = ADVISOR_FOREIGN,  
	CheckFunction = GoodyHutCheck,				    
	UnitIndexFunction = nil,						
	PlotFunction = GetGoodyHutPlot,	      
	MinTurn =  1,  
	TurnsCheck = -1, 
	PesterAgainTurn = 5,
	TutorialLevel = 0,
	Concept1 = "CONCEPT_ANCIENT_RUINS",
	Concept2 = "CONCEPT_ANCIENT_RUINS_BENEFITS",
	Concept3 = "CONCEPT_MOVEMENT",	
};

------------------------------------------------------------
------------------------------------------------------------
function BuildMoreMilitaryCheck ()
	local player = GetPlayer();
	if (player == nil) then
		dprint("Error - player index not correct");		
		return INACTIVE;	
	end	
	
	if (Game.GetGameTurn() > 50 or player:GetNumMilitaryUnits() >= 2) then
		return COMPLETE;
	end
	
	-- see if any city has a military unit queued next
	for v in player:Cities() do
		local UnitType = v:GetProductionUnit();
		if (UnitType >= 0 and GameInfo.Units[UnitType].Combat > 0) then
			dprint("City has military unit queued");
			return INACTIVE;
		end
	end
	
	-- see if any cities are done with production
	for v in player:Cities() do
		if (v:GetProduction() == 0) then
			if (player:GetNumMilitaryUnits() <= player:GetNumCities()) then
				return ACTIVE;
			end		
		end
	end
		
	return INACTIVE;
end

Tutorial { 
	ID = "BUILD_MORE_MILITARY",	    
	ActivateButtonText = false,	
	Advisor = ADVISOR_MILITARY, 
	CheckFunction = BuildMoreMilitaryCheck,		    
	UnitIndexFunction = nil,						
	PlotFunction = nil,				      
	MinTurn = 15, 
	MinTurnTime = -1, 
	TurnsCheck =  0, 
	PesterAgainTurn = -1,
	TutorialLevel = 0,
	Concept1 = "CONCEPT_CITY_CONSTRUCTING_UNITS",
	Concept2 = "CONCEPT_COMBAT",
	Concept3 = "CONCEPT_VICTORY_VICTORY_DOMINATION",	
};

------------------------------------------------------------
------------------------------------------------------------
function PeopleUnhappyCheck ()
	local player = GetPlayer();
	if (player == nil) then
		dprint("Error - player index not correct");		
		return;	
	end

	if (not player:IsEmpireUnhappy()) then
		return INACTIVE;
	end

	local bHasWorker = false;
	for v in player:Units() do
		if (IsStrictlyWorker(v)) then
			bHasWorker = true;
		end
	end
	
	if (not bHasWorker) then
		return INACTIVE;
	end

	local plot = player:GetUnimprovedAvailableLuxuryResource();
	if (plot == nil) then
		return INACTIVE;
	end
		
	return ACTIVE;
end

function GetLuxuryResourcePlot ()
	local player = GetPlayer();
	if (player == nil) then
		dprint("Error - player index not correct");		
		return;	
	end
	
	return player:GetUnimprovedAvailableLuxuryResource();
end

Tutorial { 
	ID = "HAPPINESS_RESOURCE",		
	ActivateButtonText = true,  
	Advisor = ADVISOR_ECONOMIC, 
	CheckFunction = PeopleUnhappyCheck,			    
	UnitIndexFunction = nil,						
	PlotFunction = GetLuxuryResourcePlot, 
	MinTurn =  1, 
	MinTurnTime = -1, 
	TurnsCheck =  0, 
	PesterAgainTurn = 20, 
	TutorialLevel = 1,
	Concept1 = "CONCEPT_HAPPINESS_UNHAHPPY",
	Concept2 = "CONCEPT_RESOURCES_LUXURY",
	Concept3 = "CONCEPT_FOOD_AND_CITY_GROWTH_UNHAPPINESS",	
};

------------------------------------------------------------
------------------------------------------------------------
function PeopleVeryUnhappyCheck ()
	local player = GetPlayer();
	if (player == nil) then
		dprint("Error - player index not correct");		
		return INACTIVE;	
	end

	if (player:IsEmpireVeryUnhappy()) then
		return ACTIVE;
	end

	return INACTIVE;
end

Tutorial { 
	ID = "VERY_UNHAPPY",		
	ActivateButtonText = false,  
	Advisor = ADVISOR_ECONOMIC, 
	CheckFunction = PeopleVeryUnhappyCheck,			    
	UnitIndexFunction = nil,				
	PlotFunction = nil, 
	MinTurn =  1, 
	MinTurnTime = -1, 
	TurnsCheck =  0, 
	PesterAgainTurn = 20, 
	TutorialLevel = 1,
	Concept1 = "CONCEPT_HAPPINESS_VERY_UNHAPPY",
	Concept2 = "CONCEPT_RESOURCES_LUXURY",
	Concept3 = "CONCEPT_FOOD_AND_CITY_GROWTH_UNHAPPINESS",	
};

------------------------------------------------------------
------------------------------------------------------------
local iLastTurnNumMinorsMet = 0;
if (GetPlayer()) then
	iLastTurnNumMinorsMet = GetPlayer():GetNumMinorCivsMet();
end

function MetFirstCityStateCheck ()
	dprint("MetFirstCityStateCheck begin");
	dprint("iLastTurnNumMinorsMet: " .. iLastTurnNumMinorsMet);
	dprint("player:GetNumMinorCivsMet(): " .. GetPlayer():GetNumMinorCivsMet());
	dprint("MetFirstCityStateCheck end");

	local player = GetPlayer();
	if (player == nil) then
		dprint("Error - player index not correct");		
		return INACTIVE;	
	end

	if (iLastTurnNumMinorsMet > 0) then
		return COMPLETE;
	end

	if (player:GetNumMinorCivsMet() == 0) then
		return INACTIVE;
	end

	return ACTIVE;
end

function GetCityStatePlot ()

	local player = GetPlayer();
	if (player == nil) then
		dprint("Error - player index not correct");		
		return nil;	
	end
	
	local pTeam = Teams[player:GetTeam()];

	local pMinorCiv = nil;

	-- Loop through all the Majors the active player knows
	for iPlayerLoop = GameDefines.MAX_MAJOR_CIVS, GameDefines.MAX_CIV_PLAYERS-1, 1 do
		local pOtherPlayer = Players[iPlayerLoop];
		local iOtherTeam = pOtherPlayer:GetTeam();
		-- Valid player? - Can't be us, and has to be alive
		if (player:GetTeam() ~= iOtherTeam and pOtherPlayer:IsAlive()) then			
			-- Met this player?
			if (pTeam:IsHasMet(iOtherTeam)) then
				pMinorCiv = pOtherPlayer;
			end
		end
	end
	
	-- return capital plot if we can see it
	local pCapitalCity = pMinorCiv:GetCapitalCity();
	if (pCapitalCity) then
		local pCapitalCityPlot = pCapitalCity:Plot();
		if (pCapitalCityPlot:IsRevealed(player:GetTeam())) then
			return pCapitalCityPlot;
		end
	end
	
	-- return unit plot if we can see it
	for i, v in pMinorCiv:Units() do
		local pUnitPlot = v:GetPlot();
		if (pUnitPlot:IsRevealed(player:GetTeam())) then
			return pUnitPlot;
		end
	end

	-- return tile if we can see it
	return pMinorCiv:GetPlayerVisiblePlot(player);
end

Tutorial { 
	ID = "CITY_STATES_INTRO",			
	ActivateButtonText = true,  
	Advisor = ADVISOR_FOREIGN,  
	CheckFunction = MetFirstCityStateCheck,		    
	UnitIndexFunction = nil,						
	PlotFunction = GetCityStatePlot,	  
	MinTurn =  1, 
	MinTurnTime = -1, 
	TurnsCheck =  0, 
	PesterAgainTurn = -1,
	TutorialLevel = 1,
	Concept1 = "CONCEPT_CITY_STATE",
	Concept2 = "CONCEPT_CITY_STATE_COMMUNICATION",
	Concept3 = "CONCEPT_DIPLOMACY_CITY_STATE",	
};

------------------------------------------------------------
------------------------------------------------------------
function CityStateQuestCheck ()
	local player = GetPlayer();
	if (player == nil) then
		dprint("Error - player index not correct");		
		return INACTIVE;	
	end

	local pTeam = Teams[player:GetTeam()];

	for iPlayerLoop = 0, GameDefines.MAX_CIV_PLAYERS-1, 1 do			
		pOtherPlayer = Players[iPlayerLoop];
		iOtherTeam = pOtherPlayer:GetTeam();
		
		if (pOtherPlayer:IsMinorCiv() and pOtherPlayer:IsAlive() and not pTeam:IsAtWar(iOtherTeam)) then
			if (pTeam:IsHasMet(iOtherTeam)) then
				local iQuest = pOtherPlayer:GetActiveQuestForPlayer(player:GetID());
				if ((type(iQuest) == "number" and  iQuest >= 0) or
					(type(iQuest) == "boolean" and iQuest == true)) then
					return ACTIVE;
				end
				
				if (pOtherPlayer:GetTurnsSinceThreatenedByBarbarians() >= 0) then
					return ACTIVE;
				end
				
				if (pOtherPlayer:IsMinorWarQuestWithMajorActive()) then
					return ACTIVE;
				end
			end
		end
	end		

	return INACTIVE;
end

function GetCityStateQuestPlot ()

	local player = GetPlayer();
	if (player == nil) then
		dprint("Error - player index not correct");		
		return nil;	
	end
	
	local pTeam = Teams[player:GetTeam()];

	local iResultPlayer = -1
	for iPlayerLoop = 0, GameDefines.MAX_CIV_PLAYERS-1, 1 do		
		pOtherPlayer = Players[iPlayerLoop];
		iOtherTeam = pOtherPlayer:GetTeam();
		
		if (pOtherPlayer:IsMinorCiv() and pOtherPlayer:IsAlive() and not pTeam:IsAtWar(iOtherTeam)) then
			if (pTeam:IsHasMet(iOtherTeam)) then
				local iQuest = pOtherPlayer:GetActiveQuestForPlayer(player:GetID());
				if (iQuest) then
					iResultPlayer = iPlayerLoop;
				end
				
				if (pOtherPlayer:GetTurnsSinceThreatenedByBarbarians() >= 0) then
					iResultPlayer = iPlayerLoop;
				end
				
				if (pOtherPlayer:IsMinorWarQuestWithMajorActive()) then
					iResultPlayer = iPlayerLoop;
				end
			end
		end
		
		if (iResultPlayer ~= -1) then
			iPlayerLoop = GameDefines.MAX_CIV_PLAYERS;
		end
	end		
	
	if (iResultPlayer == -1) then
		return nil;
	end
	
	local pTeam = Teams[player:GetTeam()];

	local pMinorCiv = Players[iResultPlayer];
	
	-- return capital plot if we can see it
	local pCapitalCity = pMinorCiv:GetCapitalCity();
	if (pCapitalCity) then
		local pCapitalCityPlot = pCapitalCity:Plot();
		if (pCapitalCityPlot:IsRevealed(player:GetTeam())) then
			return pCapitalCityPlot;
		end
	end
	
	-- return unit plot if we can see it
	for i, v in pMinorCiv:Units() do
		local pUnitPlot = v:GetPlot();
		if (pUnitPlot:IsRevealed(player:GetTeam())) then
			return pUnitPlot;
		end
	end

	-- return tile if we can see it
	return pMinorCiv:GetPlayerVisiblePlot(player);
end

Tutorial { 
	ID = "CITY_STATES_QUEST",			
	ActivateButtonText = true,  
	Advisor = ADVISOR_FOREIGN,
	CheckFunction = CityStateQuestCheck,		    
	UnitIndexFunction = nil,						
	PlotFunction = GetCityStateQuestPlot,
	MinTurn =  1, 
	MinTurnTime = -1, 
	TurnsCheck =  0, 
	PesterAgainTurn = -1,
	TutorialLevel = 1,
	Concept1 = "CONCEPT_CITY_STATE_MISSIONS",
	Concept2 = "CONCEPT_CITY_STATE",
	Concept3 = "CONCEPT_CITY_STATE_INFLUENCE",	
};

------------------------------------------------------------
------------------------------------------------------------
function BesiegedCityCheck ()
	dprint("BesiegedCityCheck begin");

	local player = GetPlayer();
	if (player == nil) then
		dprint("Error - player index not correct");		
		dprint("BesiegedCityCheck end - INACTIVE");
		return INACTIVE;	
	end

	for v in player:Cities() do
		local pPlot = v:Plot();
		if (pPlot) then
			local iRange = 1;
			local iX = pPlot:GetX();
			local iY = pPlot:GetY();

			for iDX = -iRange, iRange, 1 do
				for iDY = -iRange, iRange, 1 do
					local pTargetPlot = Map.GetPlotXY(iX, iY, iDX, iDY);
					if (not pTargetPlot:IsWater() and pTargetPlot:IsVisibleEnemyUnit(player:GetID())) then
						return ACTIVE;
					end
				end
			end
		end
	end	
		
	return INACTIVE;
end

function GetBesiegedCityPlot ()
	local player = GetPlayer();
	if (player == nil) then
		return nil;	
	end

	for v in player:Cities() do
		local pPlot = v:Plot();
		if (pPlot) then
			local iRange = 1;
			local iX = pPlot:GetX();
			local iY = pPlot:GetY();

			for iDX = -iRange, iRange, 1 do
				for iDY = -iRange, iRange, 1 do
					local pTargetPlot = Map.GetPlotXY(iX, iY, iDX, iDY);
					if (not pTargetPlot:IsWater() and pTargetPlot:IsVisibleEnemyUnit(player:GetID())) then
						return pPlot;
					end
				end
			end
		end
	end		
	
	return nil;
end

Tutorial { 
	ID = "CITY_UNDER_ATTACK",			
	ActivateButtonText = true,  
	Advisor = ADVISOR_MILITARY, 
	CheckFunction = BesiegedCityCheck,              
	UnitIndexFunction = nil,                        
	PlotFunction = GetBesiegedCityPlot,   
	MinTurn =  1, 
	MinTurnTime = -1, 
	TurnsCheck = 0, 
	PesterAgainTurn = -1,
	TutorialLevel = 1,
	Concept1 = "CONCEPT_CITY_COMBAT",
	Concept2 = "CONCEPT_CITY_DEFENDING",
	Concept3 = "CONCEPT_CITY_CONSTRUCTING_UNITS",	
};

------------------------------------------------------------
------------------------------------------------------------
function MilitaryUpkeepCheck ()
	local player = GetPlayer();
	if (player == nil) then
		return INACTIVE;	
	end

	if (player:GetNumCities() <= 0) then
		return INACTIVE;
	end

	if (player:GetNumUnitsOutOfSupply() > 0) then
		return ACTIVE;
	end

	return INACTIVE;
end

Tutorial { 
	ID = "MILITARY_UPKEEP",           
	ActivateButtonText = false, 
	Advisor = ADVISOR_ECONOMIC, 
	CheckFunction = MilitaryUpkeepCheck,            
	UnitIndexFunction = nil,                        
	PlotFunction = nil,                   
	MinTurn =  1, 
	MinTurnTime = -1, 
	TurnsCheck =  0, 
	PesterAgainTurn = -1,
	TutorialLevel = 1,	
	Concept1 = "CONCEPT_GOLD_UNIT_BUILDING_MAINTENANCE",
	Concept2 = "CONCEPT_ADVISORS",
	Concept3 = "",

};

------------------------------------------------------------
------------------------------------------------------------

function UnitUpgrade ()
	local player = GetPlayer();
	if (player == nil) then
		return INACTIVE;	
	end

	for v in player:Units() do
		if (v:CanUpgradeRightNow() and not v:IsBusy()) then
			return ACTIVE;
		end
	end

	return INACTIVE;
end

function GetUpgradableUnit ()
	local player = GetPlayer();
	if (player == nil) then
		return -1;	
	end

	for v in player:Units() do
		if (v:CanUpgradeRightNow()) then
			return v:GetID();
		end
	end

	return -1;
end

Tutorial { 
	ID = "UPGRADING_UNITS",			
	ActivateButtonText = true,  
	Advisor = ADVISOR_MILITARY, 
	CheckFunction = UnitUpgrade,					
	UnitIndexFunction = GetUpgradableUnit,          
	PlotFunction = nil,					  
	MinTurn =  1, 
	MinTurnTime = -1, 
	TurnsCheck =  0, 
	PesterAgainTurn = -1,
	TutorialLevel = 1,	
	Concept1 = "CONCEPT_GOLD_UPGRADE_UNITS",
	Concept2 = "CONCEPT_UNITS_COMBAT_UNITS",
	Concept3 = "",

};		   

------------------------------------------------------------
------------------------------------------------------------

function CityCombatCheck ()
	local player = GetPlayer();
	if (player == nil) then
		dprint("Error - player index not correct");		
		return INACTIVE;	
	end

	for v in player:Units() do
		if (v:IsCombatUnit()) then
			if (v:IsEnemyInMovementRange(false, true)) then
				return ACTIVE;
			end
		end
	end

	return INACTIVE;
end

function GetNearEnemyCityUnit ()
	local player = GetPlayer();
	if (player == nil) then
		dprint("Error - player index not correct");		
		return -1;	
	end

	for v in player:Units() do
		if (v:IsCombatUnit()) then
			if (v:IsEnemyInMovementRange(false, true)) then
				return v:GetID();
			end
		end
	end

	return -1;	
end

Tutorial { 
	ID =  "ATTACKING_CITY",			
	ActivateButtonText = true,	
	Advisor = ADVISOR_MILITARY, 
	CheckFunction = CityCombatCheck,				
	UnitIndexFunction = GetNearEnemyCityUnit,		
	PlotFunction = nil,				      
	MinTurn =  1, 
	MinTurnTime = -1, 
	TurnsCheck =  0, 
	PesterAgainTurn = -1,
	TutorialLevel = 1,	
	Concept1 = "CONCEPT_CITY_COMBAT",
	Concept2 = "CONCEPT_CITY_ATTACKING_MELEE",
	Concept3 = "CONCEPT_CITY_CONQUERING",

};

------------------------------------------------------------
------------------------------------------------------------

function WereRichCheck ()
	local player = GetPlayer();
	if (player == nil) then
		dprint("Error - player index not correct");		
		return INACTIVE;	
	end

	if (player:GetGold() >= 400) then
		return ACTIVE;
	end

	return INACTIVE;
end

Tutorial { 
	ID = "LOTS_OF_MONEY",			    
	ActivateButtonText = false,	
	Advisor = ADVISOR_ECONOMIC, 
	CheckFunction = WereRichCheck,					
	UnitIndexFunction = nil,						
	PlotFunction = nil,					  
	MinTurn =  1, 
	MinTurnTime = -1, 
	TurnsCheck =  0, 
	PesterAgainTurn = -1,
	TutorialLevel = 1,	
	Concept1 = "CONCEPT_GOLD_SPENDING", 
	Concept2 = "CONCEPT_DIPLOMACY",
	Concept3 = "CONCEPT_CITY_PURCHASEITEM",

};

------------------------------------------------------------
------------------------------------------------------------
local pNaturalWonderPlot = nil;
function DiscoveredNaturalWonder (i, j)
	local iX, iY = ToGridFromHex(i, j);
	pNaturalWonderPlot = Map.GetPlot(iX, iY);
end
Events.NaturalWonderRevealed.Add( DiscoveredNaturalWonder );


function DiscoveredNaturalWonderCheck ()
	if (pNaturalWonderPlot) then
		return ACTIVE;
	end
	
	return INACTIVE;
end

function DiscoveredNaturalWonderPlot ()
	return pNaturalWonderPlot;
end

Tutorial { 
	ID = "DISCOVERED_NATURAL_WONDER",		
	ActivateButtonText = true,	
	Advisor = ADVISOR_FOREIGN, 
	CheckFunction = DiscoveredNaturalWonderCheck,   
	UnitIndexFunction = nil, 
	PlotFunction = DiscoveredNaturalWonderPlot,				      
	MinTurn =  0, 
	MinTurnTime = -1, 
	TurnsCheck = -1, 
	PesterAgainTurn = -1,
	TutorialLevel = 1,	
	Concept1 = "CONCEPT_TERRAIN_FEATURE_NATURAL_WONDER",
	Concept2 = "CONCEPT_CULTURE_GAINING",
	Concept3 = "CONCEPT_HAPPINESS",
};

------------------------------------------------------------
------------------------------------------------------------
function GetEngageDiplomacyPlayer ()
	local pPlayer = GetPlayer();
	local pTeam = Teams[pPlayer:GetTeam()];
	
	-- Loop through all the players the active player knows
	for iPlayerLoop = 0, GameDefines.MAX_CIV_PLAYERS-1, 1 do
		local pOtherPlayer = Players[iPlayerLoop];
		local iOtherTeam = pOtherPlayer:GetTeam();
		-- Valid player? - Can't be us, and has to be alive
		if (pPlayer:GetTeam() ~= iOtherTeam and pOtherPlayer:IsAlive()) then			
			-- Met this player?
			if (pTeam:IsHasMet(iOtherTeam)) then
				for v in pOtherPlayer:Cities() do
					local pPlot = v:Plot();
					if (pPlot:IsRevealed(pPlayer:GetTeam())) then
						return pOtherPlayer;
					end
				end	
			end
		end
	end
end

function HowToEngageDiplomacyCheck ()
	if (GetEngageDiplomacyPlayer() ~= nil) then
		return ACTIVE;
	end
		
	return INACTIVE;
end

function HowToEngageDiplomacyPlot ()
	local pPlayer = GetPlayer();
	
	local pOtherPlayer = GetEngageDiplomacyPlayer();
	if (pOtherPlayer) then
		for v in pOtherPlayer:Cities() do
			local pPlot = v:Plot();
			if (pPlot:IsRevealed(pPlayer:GetTeam())) then
				return pPlot;
			end
		end	
	end
		
	return nil;
end

Tutorial { 
	ID = "HOW_TO_ENGAGE_DIPLOMACY",		
	ActivateButtonText = true,	
	Advisor = ADVISOR_FOREIGN, 
	CheckFunction = HowToEngageDiplomacyCheck,   
	UnitIndexFunction = nil, 
	PlotFunction = HowToEngageDiplomacyPlot,				      
	MinTurn =  0, 
	MinTurnTime = -1, 
	TurnsCheck = -1, 
	PesterAgainTurn = -1,
	TutorialLevel = 1,	
	Concept1 = "CONCEPT_CITY_STATE_COMMUNICATION",
	Concept2 = "CONCEPT_VICTORY_VICTORY_DIPLOMATIC",
	Concept3 = "CONCEPT_DIPLOMACY",
};

------------------------------------------------------------
------------------------------------------------------------
function EndTurnCheck ()
	if (Game.GetGameTurn() > 0) then
		return COMPLETE;
	end

	local pPlayer = GetPlayer();
	if (pPlayer:GetEndTurnBlockingType() == EndTurnBlockingTypes.NO_ENDTURN_BLOCKING_TYPE) then
		return ACTIVE;
	end
	
	return INACTIVE;
end

Tutorial {
	ID = "END_TURN";
	ActivateButtonText = false;
	Advisor = ADVISOR_SCIENCE,
	CheckFunction = EndTurnCheck;
	UnitIndexFunction = nil;
	PlotFunction = nil;
	MinTurn = 0;
	MinTurnTime = 1;
	TurnsCheck = -1;
	PesterAgainTurn = -1;
	TutorialLevel = 0;
	Concept1 = "";
	Concept2 = "";
	Concept3 = "";
}

------------------------------------------------------------
------------------------------------------------------------
function HowToWinCheck ()
	if (Game.GetStartEra() == 0) then
		local pPlayer = GetPlayer();
		if (pPlayer:GetCurrentEra() == 3) then
			return ACTIVE;
		end
	else
		if (Game.GetGameTurn() - Game.GetStartTurn() > 10) then
			return ACTIVE;	
		end
	end
		
	return INACTIVE;
end

Tutorial {
	ID = "WINNING_GAME";
	ActivateButtonText = false;
	Advisor = ADVISOR_SCIENCE,
	CheckFunction = HowToWinCheck;
	UnitIndexFunction = nil;
	PlotFunction = nil;
	MinTurn = 0;
	MinTurnTime = -1;
	TurnsCheck = -1;
	PesterAgainTurn = -1;
	TutorialLevel = 1;
	Concept1 = "CONCEPT_VICTORY";
	Concept2 = "CONCEPT_VICTORY_DEFEAT";
	Concept3 = "CONCEPT_VICTORY_END_OF_TIME";
}

------------------------------------------------------------
------------------------------------------------------------

local CAMP_TYPE = GameInfoTypes["IMPROVEMENT_BARBARIAN_CAMP"];
local pBarbarianCampPlot = nil;

function OnImprovementCreated (HPosX, HPosY, CultureType, ContinentType, PlayerType, engineImprovementTypeDoNotUse, ImprovementType, engineResourceTypeDoNotUse, RawResourceType, ImprovementEra, ImprovementState)
	--print("OnImprovementCreated");
	--print("HPosX: " .. HPosX);
	--print("HPosY: " .. HPosY);
	--print("CultureType: " .. CultureType);
	--print("PlayerType: " .. PlayerType);
	--print("ImprovementType: " .. ImprovementType);
	--print("RawResourceType: " .. RawResourceType);
	--print("ImprovementEra: " .. ImprovementEra);
	--print("ImprovementState: " .. ImprovementState);
	--print("------------------");
		
	local iX, iY = ToGridFromHex( HPosX, HPosY );
	local pPlot = Map.GetPlot(iX, iY);

	--print("X: " .. iX);
	--print("Y: " .. iY);
	
	if (ImprovementType == CAMP_TYPE) then
		pBarbarianCampPlot = pPlot;
	end	
end
Events.SerialEventImprovementCreated.Add( OnImprovementCreated );

function GetBarbarianCampPlot ()
	return pBarbarianCampPlot;
end

function BarbarianCampCheck ()
	if (pBarbarianCampPlot and pBarbarianCampPlot:HasBarbarianCamp()) then
		return ACTIVE;
	end
	
	return INACTIVE;
end

Tutorial {
	ID = "BARBARIAN_CAMP";
	ActivateButtonText = true;
	Advisor = ADVISOR_MILITARY,
	CheckFunction = BarbarianCampCheck;
	UnitIndexFunction = nil;
	PlotFunction = GetBarbarianCampPlot;
	MinTurn = 0;
	MinTurnTime = -1;
	TurnsCheck = -1;
	PesterAgainTurn = -1;
	TutorialLevel = 1;
	Concept1 = "CONCEPT_BARBARIANS_ENCAMPMENTS";
	Concept2 = "CONCEPT_BARBARIANS";
	Concept3 = "CONCEPT_GOLD_BARBARIAN_CAMPS";
}

------------------------------------------------------------
------------------------------------------------------------
function HappinessCheck ()
	local pPlayer = GetPlayer();

	for v in pPlayer:Cities() do
		if (v:GetPopulation() >= 3) then
			return ACTIVE;
		end
	end	
	
	return INACTIVE;
end

Tutorial {
	ID = "HAPPINESS";
	ActivateButtonText = false;
	Advisor = ADVISOR_ECONOMIC,
	CheckFunction = HappinessCheck;
	UnitIndexFunction = nil;
	PlotFunction = nil;
	MinTurn = 0;
	MinTurnTime = -1;
	TurnsCheck = -1;
	PesterAgainTurn = -1;
	TutorialLevel = 1;
	Concept1 = "CONCEPT_HAPPINESS";
	Concept2 = "CONCEPT_FOOD_AND_CITY_GROWTH_UNHAPPINESS";
	Concept3 = "CONCEPT_RESOURCES_LUXURY";
}

------------------------------------------------------------
------------------------------------------------------------
function ExcessLuxuryResourceCheck ()
	local pPlayer = GetPlayer();
	
	local luxoryResources = {};
	
	local i = 1;
	for resource in GameInfo.Resources("ResourceClassType = 'RESOURCECLASS_LUXURY'") do
		luxoryResources[i] = resource.ID;
		i = i + 1;
	end
	
	local whatWeHave = {};
	local whatWeCanTrade = {};
	for i,v in ipairs(luxoryResources) do
		local amount = pPlayer:GetNumResourceAvailable(iResourceLoop, false);
		whatWeHave[v] = amount;
		if(amount > 1) then
			table.insert(whatWeCanTrade, v);
		end
	end
	
	if(#whatWeCanTrade > 0) then
		local team = pPlayer:GetTeam();
		local pTeam = Teams[team];

		for iPlayerLoop = 0, GameDefines.MAX_MAJOR_CIVS, 1 do

			local pOtherPlayer = Players[iPlayerLoop];
			local iOtherTeam = pOtherPlayer:GetTeam();

			-- Valid player? - Can't be us, and has to be alive
			if (team ~= iOtherTeam and pOtherPlayer:IsAlive()) then			

				-- Met this player?
				if (pTeam:IsHasMet(iOtherTeam) and not pTeam:IsAtWar(iOtherTeam)) then
					
					for i, tradeableResource in ipairs(whatWeCanTrade) do
						-- if they have none of the resource
						if (pOtherPlayer:GetNumResourceAvailable(tradeableResource, true) == 0) then
							--print("They don't have our resource");
							-- find a resource we need
							for li, luxoryResource in ipairs(luxoryResources) do
								if(whatWeHave[luxoryResource] == 0) then
									if(pOtherPlayer:GetNumResourceAvailable(tradeableResource, false) > 0) then
										return ACTIVE;
									end
								end 		
							end
						end
					end		
				end
			end
		end
	end
	
	return INACTIVE;
end

Tutorial {
	ID = "EXCESS_LUXURY_RESOURCE";
	ActivateButtonText = false;
	Advisor = ADVISOR_FOREIGN,
	CheckFunction = ExcessLuxuryResourceCheck;
	UnitIndexFunction = nil;
	PlotFunction = nil;
	MinTurn = 0;
	MinTurnTime = -1;
	TurnsCheck = 1;
	PesterAgainTurn = -1;
	TutorialLevel = 1;
	Concept1 = "CONCEPT_RESOURCES_LUXURY";
	Concept2 = "CONCEPT_DIPLOMACY_TRADE";
	Concept3 = "CONCEPT_DIPLOMACY_RESOURCES";
}

------------------------------------------------------------
------------------------------------------------------------

function BankruptCheck ()
	local pPlayer = GetPlayer();
	if (pPlayer:GetGold() < 0) then
		return ACTIVE;
	end
	
	return INACTIVE;
end

Tutorial {
	ID = "BANKRUPTCY";
	ActivateButtonText = false;
	Advisor = ADVISOR_ECONOMIC,
	CheckFunction = BankruptCheck;
	UnitIndexFunction = nil;
	PlotFunction = nil;
	MinTurn = 0;
	MinTurnTime = -1;
	TurnsCheck = -1;
	PesterAgainTurn = -1;
	TutorialLevel = 1;
	Concept1 = "CONCEPT_GOLD_RUNNING_DEFICIT";
	Concept2 = "CONCEPT_GOLD_WHERE_TO_GET";
	Concept3 = "";
}

------------------------------------------------------------
------------------------------------------------------------
function GetCSFriendPlot ()
	local pPlayer = GetPlayer();
	local pTeam = Teams[pPlayer:GetTeam()];
	
	for iPlayerLoop = GameDefines.MAX_MAJOR_CIVS, GameDefines.MAX_CIV_PLAYERS - 1, 1 do	
		local pOtherPlayer = Players[iPlayerLoop];
		local iOtherTeam = pOtherPlayer:GetTeam();
		if (pPlayer:GetTeam() ~= iOtherTeam and pOtherPlayer:IsAlive() and pOtherPlayer:IsFriends()) then
			for v in pOtherPlayer:Cities() do
				local pPlot = v:Plot();
				if (pPlot:IsRevealed(pPlayer:GetTeam())) then
					return pPlot;
				end
			end
		end
	end
	
	return nil;
end


function MadeCSFriendCheck ()
	local pPlot = GetCSFriendPlot();
	if (pPlot) then
		return ACTIVE;
	end
	
	return INACTIVE;
end

Tutorial {
	ID = "CITY_STATE_FRIEND";
	ActivateButtonText = true;
	Advisor = ADVISOR_FOREIGN;
	CheckFunction = MadeCSFriendCheck;
	UnitIndexFunction = nil;
	PlotFunction = GetCSFriendPlot;
	MinTurn = 0;
	MinTurnTime = -1;
	TurnsCheck = -1;
	PesterAgainTurn = -1;
	TutorialLevel = 1;
	Concept1 = "CONCEPT_CITY_STATE_INFLUENCE_LEVELS_FRIENDS";
	Concept2 = "CONCEPT_CITY_STATE_INFLUENCE";
	Concept3 = "CONCEPT_CITY_STATE_COMMUNICATION";
}

------------------------------------------------------------
------------------------------------------------------------
function TutorialBuildCityCheatCheck ()
	local player = GetPlayer();
	if (player == nil) then
		dprint("Error - player index not correct");		
		return INACTIVE;
	end

	if (player:GetNumCities() > 0) then
		return ACTIVE;
	end
	
	return INACTIVE;
end

function TutorialBuildCityCheatPlot ()
	local player = GetPlayer();
	for v in player:Cities() do
		return v:Plot();
	end
end

Tutorial { 
	ID = "TUTORIAL_BUILD_CITY_CHEAT",		
	ActivateButtonText = true,	
	Advisor = ADVISOR_ECONOMIC, 
	CheckFunction = TutorialBuildCityCheatCheck,   
	UnitIndexFunction = nil, 
	PlotFunction = TutorialBuildCityCheatPlot,				      
	MinTurn =  0, 
	MinTurnTime = -1, 
	TurnsCheck = -1, 
	PesterAgainTurn = -1,
	TutorialLevel = 1,	
	Concept1 = "CONCEPT_CITIES",
	Concept2 = "",
	Concept3 = "",
};
------------------------------------------------------------
------------------------------------------------------------
function Tutorial1MovementIntro ()
	local player = GetPlayer();
	if (player == nil) then
		dprint("Error - player index not correct");		
		return INACTIVE;
	end

	if (Game.GetGameTurn() > 0) then
		return INVALID;
	end
	
	return ACTIVE;
end

Tutorial { 
	ID = "TUTORIAL_1_INTRO_MOVEMENT_AND_EXPLORATION",		
	ActivateButtonText = false,	
	Advisor = ADVISOR_SCIENCE, 
	CheckFunction = Tutorial1MovementIntro,   
	UnitIndexFunction = nil, 
	PlotFunction = nil,				      
	MinTurn =  0, 
	MinTurnTime = 0, 
	TurnsCheck = -1, 
	PesterAgainTurn = -1,
	TutorialLevel = 999,	
	Concept1 = "CONCEPT_MOVEMENT",
	Concept2 = "CONCEPT_FOG_OF_WAR",
	Concept3 = "CONCEPT_TERRAIN",

};

------------------------------------------------------------
------------------------------------------------------------
function Tutorial2FoundCitiesIntro ()
	local player = GetPlayer();
	if (player == nil) then
		dprint("Error - player index not correct");		
		return INACTIVE;
	end

	if (Game.GetGameTurn() > 0) then
		return INVALID;
	end
	
	return ACTIVE;
end

Tutorial {
	ID = "TUTORIAL_2_INTRO_FOUND_CITIES",
	ActivateButtonText = false,	
	Advisor = ADVISOR_ECONOMIC, 
	CheckFunction = Tutorial2FoundCitiesIntro,   
	UnitIndexFunction = nil, 
	PlotFunction = nil,
	MinTurn =  0, 
	MinTurnTime = 5, 
	TurnsCheck = -1, 
	PesterAgainTurn = -1,
	TutorialLevel = 999,	
	Concept1 = "CONCEPT_CITIES",
	Concept2 = "CONCEPT_FOUNDING_CITIES",
	Concept3 = "CONCEPT_WHERE_TO_FOUND_CITIES",

};

------------------------------------------------------------
------------------------------------------------------------
function Tutorial4CombatAndConquestIntro ()
	local player = GetPlayer();
	if (player == nil) then
		dprint("Error - player index not correct");		
		return INACTIVE;
	end

	if (Game.GetGameTurn() > 0) then
		return INVALID;
	end
	
	return ACTIVE;
end

Tutorial { 
	ID = "TUTORIAL_4_INTRO_COMBAT_AND_CONQUERING",		
	ActivateButtonText = false,	
	Advisor = ADVISOR_MILITARY, 
	CheckFunction = Tutorial4CombatAndConquestIntro,   
	UnitIndexFunction = nil, 
	PlotFunction = nil,
	MinTurn =  0, 
	MinTurnTime = 0, 
	TurnsCheck = -1, 
	PesterAgainTurn = -1,
	TutorialLevel = 0,	
	Concept1 = "CONCEPT_COMBAT",
	Concept2 = "CONCEPT_BARBARIANS_ENCAMPMENTS",
	Concept3 = "CONCEPT_CITY_CONQUERING",

};

------------------------------------------------------------
------------------------------------------------------------
function Tutorial5DiplomacyIntro ()
	local player = GetPlayer();
	if (player == nil) then
		dprint("Error - player index not correct");		
		return INACTIVE;
	end

	if (Game.GetGameTurn() > 0) then
		return INVALID;
	end
	
	return ACTIVE;
end

Tutorial { 
	ID = "TUTORIAL_5_INTRO_DIPLOMACY",		
	ActivateButtonText = false,	
	Advisor = ADVISOR_FOREIGN, 
	CheckFunction = Tutorial5DiplomacyIntro,   
	UnitIndexFunction = nil, 
	PlotFunction = nil,
	MinTurn =  0, 
	MinTurnTime = 0, 
	TurnsCheck = -1, 
	PesterAgainTurn = -1,
	TutorialLevel = 0,	
	Concept1 = "CONCEPT_RESOURCES",
	Concept2 = "CONCEPT_DIPLOMACY",
	Concept3 = "CONCEPT_CITY_STATE",
};


------------------------------------------------------------
------------------------------------------------------------

function TutorialChecksEndTurn ()

	local player = GetPlayer();
	if (player == nil) then
		return;	
	end

	if (not EverHealedUnit) then	
		for v in player:Units() do
			if (v:GetActivityType() == ActivityTypes.ACTIVITY_HEAL) then
				EverHealedUnit = true;
			end
		end
	end
	
	if (not bEverCityBombard) then
		for v in player:Cities() do
			if (v:HasPerformedRangedStrikeThisTurn()) then
				bEverCityBombard = true;
			end
		end
	end
	
	if (iMaxThingsCityBuilt == 0) then
		for v in player:Cities() do
			if (v:GetNumThingsProduced() > iMaxThingsCityBuilt) then
				print("City building complete");
				iMaxThingsCityBuilt = v:GetNumThingsProduced();
			end
		end	
	end
	
	iLastTurnNumMinorsMet = GetPlayer():GetNumMinorCivsMet();
end

Tutorial {
	ID = "CITY_SCREEN",
	Advisor = ADVISOR_ECONOMIC,
	TitleText = "TXT_KEY_ADVISOR_CITY_SCREEN_DISPLAY",
	BodyText = "TXT_KEY_ADVISOR_CITY_SCREEN_BODY",
	TutorialLevel = 1,
	Modal = true,		
};

Tutorial {
	ID = "DIPLO_TRADE_SCREEN",
	Advisor = ADVISOR_FOREIGN,
	TitleText = "TXT_KEY_ADVISOR_SCREEN_LEADER_TRADE_DIALOG_DISPLAY",
	BodyText = "TXT_KEY_ADVISOR_SCREEN_LEADER_TRADE_DIALOG_BODY",
	TutorialLevel = 1,
	Modal = true,		
};

Tutorial {
	ID = "CHOOSE_TECH_SCREEN",
	Advisor = ADVISOR_SCIENCE,
	TitleText = "TXT_KEY_ADVISOR_SCREEN_TECH_CHOOSER_DISPLAY",
	BodyText = "TXT_KEY_ADVISOR_SCREEN_TECH_CHOOSER_BODY",
	ButtonPopupType = ButtonPopupTypes.BUTTONPOPUP_CHOOSETECH,
	TutorialLevel = 1,
	Modal = true,		
};

Tutorial {
	ID = "CHOOSE_PRODUCTION_SCREEN",
	Advisor = ADVISOR_ECONOMIC,
	TitleText = "TXT_KEY_ADVISOR_SCREEN_PRODUCTION_CHOOSER_DISPLAY",
	BodyText = "TXT_KEY_ADVISOR_SCREEN_PRODUCTION_CHOOSER_BODY",
	ButtonPopupType = ButtonPopupTypes.BUTTONPOPUP_CHOOSEPRODUCTION,
	TutorialLevel = 1,
	Modal = true,		
};

Tutorial {
	ID = "CHOOSE_POLICY_SCREEN",
	Advisor = ADVISOR_ECONOMIC,
	TitleText = "TXT_KEY_ADVISOR_SCREEN_SOCIAL_POLICY_DISPLAY",
	BodyText = "TXT_KEY_ADVISOR_SCREEN_SOCIAL_POLICY_BODY",
	ButtonPopupType = ButtonPopupTypes.BUTTONPOPUP_CHOOSEPOLICY,
	TutorialLevel = 1,
	Modal = true,		
};

Tutorial {
	ID = "ECONOMIC_OVERVIEW_SCREEN",
	Advisor = ADVISOR_ECONOMIC,
	TitleText = "TXT_KEY_ADVISOR_SCREEN_ECONOMIC_OVERVIEW_DISPLAY",
	BodyText = "TXT_KEY_ADVISOR_SCREEN_ECONOMIC_OVERVIEW_BODY",
	ButtonPopupType = ButtonPopupTypes.BUTTONPOPUP_ECONOMIC_OVERVIEW,
	TutorialLevel = 1,
	Modal = true,		
};

Tutorial {
	ID = "MILITARY_OVERVIEW_SCREEN",
	Advisor = ADVISOR_MILITARY,
	TitleText = "TXT_KEY_ADVISOR_SCREEN_MILITARY_OVERVIEW_DISPLAY",
	BodyText = "TXT_KEY_ADVISOR_SCREEN_MILITARY_OVERVIEW_BODY",
	ButtonPopupType = ButtonPopupTypes.BUTTONPOPUP_MILITARY_OVERVIEW,
	TutorialLevel = 1,
	Modal = true,		
};

Tutorial {
	ID = "DIPLOMATIC_OVERVIEW_SCREEN",
	Advisor = ADVISOR_ECONOMIC,
	TitleText = "TXT_KEY_ADVISOR_SCREEN_DIPLOMACY_OVERVIEW_DISPLAY",
	BodyText = "TXT_KEY_ADVISOR_SCREEN_DIPLOMACY_OVERVIEW_BODY",
	ButtonPopupType = ButtonPopupTypes.BUTTONPOPUP_DIPLOMATIC_OVERVIEW,
	TutorialLevel = 1,
	Modal = true,		
};

Tutorial {
	ID = "CITY_STATES_INTRO",
	Advisor = ADVISOR_FOREIGN,
	TitleText = "TXT_KEY_ADVISOR_CITY_STATES_INTRO_DISPLAY",
	BodyText = "TXT_KEY_ADVISOR_CITY_STATES_INTRO_BODY",
	ButtonPopupType = ButtonPopupTypes.BUTTONPOPUP_CITY_STATE_GREETING,
	TutorialLevel = 1,
	Modal = true,		
};

Tutorial {
	ID = "TECH_TREE_SCREEN",
	Advisor = ADVISOR_SCIENCE,
	TitleText = "TXT_KEY_ADVISOR_SCREEN_TECH_TREE_DISPLAY",
	BodyText = "TXT_KEY_ADVISOR_SCREEN_TECH_TREE_BODY",
	ButtonPopupType = ButtonPopupTypes.BUTTONPOPUP_TECH_TREE,
	TutorialLevel = 1,
	Modal = true,		
};
