-- the order of these determines the order which are evaluated
TutorialInfo = {
GlobalTutorialInfo["END_TURN"],
GlobalTutorialInfo["GO_TO_GOODY_HUT"],
GlobalTutorialInfo["WINNING_GAME"],
GlobalTutorialInfo["DISCOVERED_NATURAL_WONDER"],
GlobalTutorialInfo["LOST_ALL_MILITARY"],
GlobalTutorialInfo["BUILD_MORE_MILITARY"],
GlobalTutorialInfo["HAPPINESS_RESOURCE"],
GlobalTutorialInfo["VERY_UNHAPPY"],
GlobalTutorialInfo["CITY_STATES_QUEST"],
GlobalTutorialInfo["HOW_TO_ENGAGE_DIPLOMACY"],
GlobalTutorialInfo["CITY_UNDER_ATTACK"],
GlobalTutorialInfo["MILITARY_UPKEEP"],
--GlobalTutorialInfo["UPGRADING_UNITS"],
GlobalTutorialInfo["ATTACKING_CITY"],
GlobalTutorialInfo["LOTS_OF_MONEY"],
GlobalTutorialInfo["BARBARIAN_CAMP"],
GlobalTutorialInfo["HAPPINESS"],
GlobalTutorialInfo["EXCESS_LUXURY_RESOURCE"],
GlobalTutorialInfo["BANKRUPTCY"],
GlobalTutorialInfo["CITY_STATE_FRIEND"],
GlobalTutorialInfo["CITY_SCREEN"],
GlobalTutorialInfo["DIPLO_TRADE_SCREEN"],
GlobalTutorialInfo["CHOOSE_TECH_SCREEN"],
GlobalTutorialInfo["CHOOSE_PRODUCTION_SCREEN"],
GlobalTutorialInfo["CHOOSE_POLICY_SCREEN"],
GlobalTutorialInfo["ECONOMIC_OVERVIEW_SCREEN"],
GlobalTutorialInfo["MILITARY_OVERVIEW_SCREEN"],
GlobalTutorialInfo["DIPLOMATIC_OVERVIEW_SCREEN"],
GlobalTutorialInfo["CITY_STATES_INTRO"],
GlobalTutorialInfo["TECH_TREE_SCREEN"],
GlobalTutorialInfo["STARVATION"],
GlobalTutorialInfo["DIPLOMAT"],
GlobalTutorialInfo["MUSICIAN"],
GlobalTutorialInfo["MISSIONARY"],
GlobalTutorialInfo["INQUISITOR"],
};

--print("TUTORIAL SYSTEM CHECK");
--print(TutorialInfo);
----
--
--print(TutorialInfo);

function TutorialSetup ()
	-- do nothing for the main tutorial
end

function IsScenarioComplete ()
	return false;
end

function DoScenarioComplete ()

end

-- utility functions
function GetPlayer ()
	local iPlayerID = Game.GetActivePlayer();
	if (iPlayerID < 0) then
		return nil;
	end

	local player = Players[iPlayerID];
	if (player ~= nil and player:IsHuman()) then
		return player
	end;
	
	return nil;
end

function FirstFaithPointCheck()
	local player = GetPlayer();
	if (player == nil) then
		return INACTIVE;	
	end
	
	if(player:GetFaith() > 0) then
		return ACTIVE;
	end	
			
	return INACTIVE;
end

function FaithReligionAnd200PointCheck()
	local player = GetPlayer();
	if (player == nil) then
		return INACTIVE;	
	end

	if ((player:HasCreatedReligion() or Game.GetNumReligionsStillToFound() == 0) and player:GetFaith() >= 200) then
		return ACTIVE;
	end	
			
	return INACTIVE;
end


function FaithReligionAnd400PointCheck()
	local player = GetPlayer();
	if (player == nil) then
		return INACTIVE;	
	end

	if ((player:HasCreatedReligion() or Game.GetNumReligionsStillToFound() == 0) and player:GetFaith() >= 400) then
		return ACTIVE;
	end	
			
	return INACTIVE;
end

function FirstSpyCheck()
	-- Set to INACTIVE if this is the starting turn.
	local gameTurn = Game.GetGameTurn();
	local startTurn = Game.GetStartTurn();
	
	if(gameTurn <= startTurn) then
		return INACTIVE;
	end

	local player = GetPlayer();
	if (player == nil) then
		return INACTIVE;	
	end
	
	local agents = player:GetEspionageSpies();
	if(#agents ~= 0) then
		return ACTIVE;
	end
	
	return INACTIVE;
end

if(TutorialInfo ~= nil) then

	-- First Spy
	table.insert(TutorialInfo, { 
		ID = "FIRST_SPY",		
		Advisor = ADVISOR_FOREIGN,
		NotificationType = NotificationTypes.NOTIFICATION_SPY_CREATED_ACTIVE_PLAYER,   
		TutorialLevel = 2,
		ActivateButtonText = true,  
		ActivateButtonPopupType = ButtonPopupTypes.BUTTONPOPUP_ESPIONAGE_OVERVIEW, 
		Concept1 = "CONCEPT_ESPIONAGE_SPIES",
		Concept2 = "CONCEPT_ESPIONAGE_MOVING_SPIES",
	});
	
	table.insert(TutorialInfo, {
		ID = "CHOOSE_TECH_TO_STEAL",
		Advisor = ADVISOR_FOREIGN,
		ButtonPopupType = ButtonPopupTypes.BUTTONPOPUP_CHOOSE_TECH_TO_STEAL,
		TutorialLevel = 2,
		Concept1 = "CONCEPT_ESPIONAGE_STEALING_TECHS",
		Concept2 = "CONCEPT_ESPIONAGE_SURVEILLANCE",
		Concept3 = "CONCEPT_ESPIONAGE_COUNTER_INTEL",
	});
	
	table.insert(TutorialInfo, {
		ID = "ESPIONAGE_TECH_STOLEN",
		Advisor = ADVISOR_FOREIGN,
		NotificationType = NotificationTypes.NOTIFICATION_TECH_STOLEN_SPY_DETECTED;
		ActivateButtonText = true,  
		ActivateButtonPopupType = ButtonPopupTypes.BUTTONPOPUP_ESPIONAGE_OVERVIEW, 
		TutorialLevel = 2,
		Concept1 = "CONCEPT_ESPIONAGE_POLICE_STATION",
		Concept2 = "CONCEPT_ESPIONAGE_GETTING_CAUGHT",
		Concept3 = "CONCEPT_ESPIONAGE_STEALING_TECHS",
	});
	
	table.insert(TutorialInfo, {
		ID = "ESPIONAGE_CANT_STEAL_ANYMORE",
		Advisor = ADVISOR_SCIENCE,
		NotificationType = NotificationTypes.NOTIFICATION_SPY_CANT_STEAL_TECH;
		ActivateButtonText = true,  
		ActivateButtonPopupType = ButtonPopupTypes.BUTTONPOPUP_ESPIONAGE_OVERVIEW, 
		TutorialLevel = 2,
		Concept1 = "CONCEPT_ESPIONAGE_SPIES",
		Concept2 = "CONCEPT_ESPIONAGE_SURVEILLANCE",
	});
	
	table.insert(TutorialInfo, {
		ID = "ESPIONAGE_OVERVIEW",
		Advisor = ADVISOR_ECONOMIC,
		ButtonPopupType = ButtonPopupTypes.BUTTONPOPUP_ESPIONAGE_OVERVIEW,
		Modal = true,		
		TutorialLevel = 2,
	});
	
	-- First Faith Point
	table.insert(TutorialInfo, { 
		ID = "FIRST_FAITH_POINT",	
		Advisor = ADVISOR_ECONOMIC, 
		CheckFunction = FirstFaithPointCheck, 
		MinTurn = 1, 
		MinTurnTime = 0, 
		TurnsCheck = 1, 
		TutorialLevel = 2,
		Concept1 = "CONCEPT_RELIGION_PANTHEONS",
		Concept2 = "CONCEPT_RELIGION_FOUNDING_RELIGION",
		Concept3 = "CONCEPT_RELIGION_FAITH_SPENDING",
	});
	
	table.insert(TutorialInfo, {
		ID = "FOUND_PANTHEON",
		Advisor = ADVISOR_ECONOMIC,
		TitleText = "TXT_KEY_ADVISOR_RELIGION_FOUND_PANTHEON_DISPLAY",
		BodyText = "TXT_KEY_ADVISOR_RELIGION_FOUND_PANTHEON_BODY",
		NotificationType = NotificationTypes.NOTIFICATION_FOUND_PANTHEON,
		TutorialLevel = 2,
		Concept1 = "CONCEPT_RELIGION_PANTHEONS",
		Concept2 = "CONCEPT_RELIGION_FOUNDING_RELIGION",
	});
	
	table.insert(TutorialInfo, {
		ID = "FOUND_RELIGION",
		Advisor = ADVISOR_ECONOMIC,
		TitleText = "TXT_KEY_ADVISOR_RELIGION_FOUND_RELIGION_DISPLAY",
		BodyText = "TXT_KEY_ADVISOR_RELIGION_FOUND_RELIGION_BODY",
		NotificationType = NotificationTypes.NOTIFICATION_FOUND_RELIGION,
		TutorialLevel = 2,
	});
	
	table.insert(TutorialInfo, {
		ID = "RELIGION_FIRST_FAITH_PURCHASE_AVAILABLE",
		Advisor = ADVISOR_ECONOMIC, 
		CheckFunction = FirstFaithPurchaseAvailable,   
		PlotIndexFunction = nil, 
		MinTurn =  1, 
		MinTurnTime = 5, 
		TurnsCheck = 1, 
		TutorialLevel = 2,
		Concept1 = "CONCEPT_RELIGION_FAITH_SPENDING",
		Concept2 = "CONCEPT_RELIGION_BUILDINGS",
		Concept3 = "CONCEPT_RELIGION_UNITS",
	});
	
	table.insert(TutorialInfo, {
		ID = "RELIGION_OVERVIEW",
		Advisor = ADVISOR_ECONOMIC,
		ButtonPopupType = ButtonPopupTypes.BUTTONPOPUP_RELIGION_OVERVIEW,
		TutorialLevel = 2,
		Modal = true,	
	});
		
	table.insert(TutorialInfo, {
		ID = "CITY_STATE_AFRAID",
		Advisor = ADVISOR_MILITARY,
		TutorialLevel = 3,
		Concept1 = "CONCEPT_CITY_STATE_BULLYING",
		Concept2 = "CONCEPT_CITY_STATE_INLUENCE",
		Concept3 = "CONCEPT_CITY_STATE_INFLUENCE_LEVELS",
	});
	
end

-- utility functions
function GetPlayer ()
	local iPlayerID = Game.GetActivePlayer();
	if (iPlayerID < 0) then
		return nil;
	end

	local player = Players[iPlayerID];
	if (player ~= nil and player:IsHuman()) then
		return player
	end;
	
	return nil;
end

function ShouldShowGrowthAndControlTutorial()
	local player = GetPlayer();
	if (player == nil) then
		return INACTIVE;	
	end
	return ACTIVE;
end

function CheckHasTech(techType)
	local techID = GameInfo.Technologies[techType].ID;
	print("checking if player has tech type: ", techType);
	return function()
		local player = GetPlayer();
		if(player == nil) then
			return INACTIVE;
		end
		
		local team = Teams[player:GetTeam()];
		if(team == nil) then
			return INACTIVE;
		end
		
		if(team:IsHasTech(techID)) then
			return ACTIVE;
		end
	
		return INACTIVE;
	end
end


function GetUnitClassID(unitClassType)
	local unitClass = GameInfo.UnitClasses[unitClassType];
	if(unitClass ~= nil) then
		return unitClass.ID;
	end
end
	
function CreateFirstUnitOfUnitClassCheck(unitClassType)
	local unitClassID = GetUnitClassID(unitClassType);
	
	if(unitClassID ~= nil) then
		return function()
			local player = GetPlayer();
			if(player == nil) then
				return INACTIVE;
			end
			
			if(player:HasUnitOfClassType(unitClassID)) then
				return ACTIVE;
			else
				return INACTIVE;
			end
		end
	end
end

function CheckHasTourism()
	local player = GetPlayer();
	
	if(player == nil) then
		return INACTIVE;
	end
	
	if(player:GetTourism() > 0) then
		return ACTIVE;
	else
		return INACTIVE;
	end
end

function CheckCanPropose()
	local player = GetPlayer();
	
	if(player == nil) then
		return INACTIVE;
	end
	
	local playerID = player:GetID();
	
	for leagueID = 0, Game.GetNumActiveLeagues() - 1, 1 do
		local league = Game.GetLeague(leagueID);
		if(league:IsMember(playerID) and league:CanPropose(playerID)) then
			return ACTIVE;
		end
	end
	
	return INACTIVE;
end

if(TutorialInfo ~= nil) then

	-- Culture Overview
	table.insert(TutorialInfo, {
		ID = "CULTURE_OVERVIEW",
		Advisor = ADVISOR_FOREIGN,
		ButtonPopupType = ButtonPopupTypes.BUTTONPOPUP_CULTURE_OVERVIEW,
		Modal = true,		
		TutorialLevel = 3,
		Concept1 = "CONCEPT_CULTURE_TOURISM",
		Concept2 = "CONCEPT_CULTURE_TOURISM_AND_CULTURE",
		Concept3 = "CONCEPT_CULTURE_GREAT_WORKS",
	});
	
	
	local caravanUnit = GameInfo.Units["UNIT_CARAVAN"];
	if(caravanUnit ~= nil) then	
		-- First Caravan
		table.insert(TutorialInfo, { 
			ID = "FIRST_CARAVAN",		
			Advisor = ADVISOR_ECONOMIC,
			TutorialLevel = 3,
			CheckFunction = CheckHasTech(caravanUnit.PrereqTech),
			TurnTimeCheck = 2,
			Concept1 = "CONCEPT_TRADE_ROUTES",
			Concept2 = "CONCEPT_TRADE_CARAVAN",
			Concept3 = "CONCEPT_TRADE_CARGO",
		});	
	end
	
	local archUnit = GameInfo.Units["UNIT_ARCHAEOLOGIST"];
	if(archUnit ~= nil) then	
		-- Antiquity Sites
		table.insert(TutorialInfo, { 
			ID = "ARCH",		
			Advisor = ADVISOR_FOREIGN,
			TutorialLevel = 3,
			CheckFunction = CheckHasTech(archUnit.PrereqTech),
			TurnTimeCheck = 2,
			Concept1 = "CONCEPT_CULTURE_ARCH",
			Concept2 = "CONCEPT_CULTURE_ANTIQUITY",
			Concept3 = "CONCEPT_CULTURE_ARTIFACTS",
		});	
	end
	
	local checkFirstArcheologist = CreateFirstUnitOfUnitClassCheck("UNITCLASS_ARCHEOLOGIST");
	if(checkFirstArcheologist) then
		table.insert(TutorialInfo, {
			ID = "ARCHEOLOGIST",		
			Advisor = ADVISOR_ECONOMIC, 
			CheckFunction = checkFirstArcheologist,   
			UnitIndexFunction = nil, 
			MinTurn =  1, 
			TurnsCheck = 1, 
			TutorialLevel = 3,
			TurnTimeCheck = 2,
			Concept1 = "CONCEPT_CULTURE_ARCH",
			Concept2 = "CONCEPT_CULTURE_ANTIQUITY",
			Concept3 = "CONCEPT_CULTURE_ARTIFACTS",
		});
	end
	
	-- World Congress Founded
	table.insert(TutorialInfo, {
		ID = "CONGRESS_FOUNDED",
		Advisor = ADVISOR_FOREIGN,
		ButtonPopupType = ButtonPopupTypes.BUTTONPOPUP_LEAGUE_SPLASH,
		Modal = true,	
		TutorialLevel = 3,
		Concept1 = "CONCEPT_CONGRESS",
		Concept2 = "CONCEPT_CONGRESS_SESSIONS",
		Concept3 = "CONCEPT_CONGRESS_RESOLUTIONS",
	});
	
	table.insert(TutorialInfo, {
		ID = "CONGRESS_PROPOSE",
		Advisor = ADVISOR_FOREIGN,
		--NotificationType = NotificationTypes.NOTIFICATION_LEAGUE_CALL_FOR_PROPOSALS,   
		ButtonPopupType = ButtonPopupTypes.BUTTONPOPUP_LEAGUE_OVERVIEW,
		CheckFunction = CheckCanPropose,
		Modal = true,		
		TutorialLevel = 3,
		Concept1 = "CONCEPT_CONGRESS_RESOLUTIONS",
		Concept2 = "CONCEPT_CONGRESS_PROJECTS",
		Concept3 = "CONCEPT_CONGRESS_DELEGATES",
	});
	
	table.insert(TutorialInfo, {
		ID = "CONGRESS_VOTE",
		Advisor = ADVISOR_FOREIGN,
		NotificationType = NotificationTypes.NOTIFICATION_LEAGUE_CALL_FOR_VOTES,   
		Modal = true,		
		TutorialLevel = 3,
		Concept1 = "CONCEPT_CONGRESS_DELEGATES",
		Concept2 = "CONCEPT_CONGRESS_TRADING",
		Concept3 = "CONCEPT_CONGRESS_INTRIGUE",
	});
	
	table.insert(TutorialInfo, { 
		ID = "TOURISM",		
		Advisor = ADVISOR_FOREIGN,
		TutorialLevel = 3,
		TurnTimeCheck = 2,
		CheckFunction = CheckHasTourism,
		Concept1 = "CONCEPT_CULTURE_TOURISM",
		Concept2 = "CONCEPT_CULTURE_TOURISM_AND_CULTURE",
	});	
	
	table.insert(TutorialInfo, {
		ID = "CHOOSE_IDEOLOGY",
		Advisor = ADVISOR_FOREIGN,
		NotificationType = NotificationTypes.NOTIFICATION_CHOOSE_IDEOLOGY,   
		Modal = true,		
		TutorialLevel = 3,
		Concept1 = "CONCEPT_SOCIAL_POLICY_BRANCH_IDEOLOGY",
	});
	
	
	table.insert(TutorialInfo, {
		ID = "CHOOSE_ARCHAEOLOGY",
		Advisor = ADVISOR_FOREIGN,
		NotificationType = NotificationTypes.NOTIFICATION_CHOOSE_ARCHAEOLOGY,   
		Modal = true,		
		TutorialLevel = 3,
		Concept1 = "CONCEPT_CULTURE_ARTIFACTS",
		Concept2 = "CONCEPT_CULTURE_TOURISM",
		Concept3 = "",
	});

	table.insert(TutorialInfo, {
		ID = "POPULATION_GROWTH_AND_TILE_CONTROL",
		Advisor = ADVISOR_ECONOMIC,
		CheckFunction = ShouldShowGrowthAndControlTutorial,
		PlotIndexFunction = nil, 
		TutorialLevel = 2,
	});

	table.insert(TutorialInfo, {
		ID = "CITIZEN_RESEARCH",
		MinTurn =  1, 
		Advisor = ADVISOR_SCIENCE,
		NotificationType = NotificationTypes.NOTIFICATION_TECH,   
		TutorialLevel = 2,
	});

	table.insert(TutorialInfo, {
		ID = "HOW_TO_CONTROL_UNITS",
		MinTurn =  2, 
		Advisor = ADVISOR_MILITARY,
		TutorialLevel = 3,
	});

	local checkFirstGeneral= CreateFirstUnitOfUnitClassCheck("UNITCLASS_GENERAL");
	if(checkFirstGeneral) then
		table.insert(TutorialInfo, {
			ID = "USING_A_GENERAL",		
			Advisor = ADVISOR_MILITARY, 
			CheckFunction = checkFirstGeneral,   
			UnitIndexFunction = nil, 
			MinTurn =  1, 
			TurnsCheck = 1, 
			TutorialLevel = 3,
			TurnTimeCheck = 2,
		});
	end

	local checkFirstAdmiral= CreateFirstUnitOfUnitClassCheck("UNITCLASS_ADMIRAL");
	if(checkFirstGeneral) then
		table.insert(TutorialInfo, {
			ID = "USING_AN_ADMIRAL",		
			Advisor = ADVISOR_MILITARY, 
			CheckFunction = checkFirstAdmiral,   
			UnitIndexFunction = nil, 
			MinTurn =  1, 
			TurnsCheck = 1, 
			TutorialLevel = 3,
			TurnTimeCheck = 2,
		});
	end

	local checkFirstCaravan = CreateFirstUnitOfUnitClassCheck("UNITCLASS_CARAVAN");
	if(checkFirstCaravan) then
		table.insert(TutorialInfo, {
			ID = "TRADE_SPREADS_CULTURE",		
			Advisor = ADVISOR_ECONOMIC, 
			CheckFunction = checkFirstCaravan,   
			UnitIndexFunction = nil, 
			MinTurn =  1, 
			TurnsCheck = 1, 
			TutorialLevel = 3,
			TurnTimeCheck = 2,
			Concept1 = "CONCEPT_TRADE_ROUTES",
			Concept2 = "CONCEPT_TRADE_CARAVAN",
			Concept3 = "CONCEPT_TRADE_CARGO",
		});
	end

	local checkFirstCargoShip = CreateFirstUnitOfUnitClassCheck("UNITCLASS_CARGO_SHIP");
	if(checkFirstCargoShip) then
		table.insert(TutorialInfo, {
			ID = "TRADE_SPREADS_CULTURE_BOAT",		
			Advisor = ADVISOR_ECONOMIC, 
			CheckFunction = checkFirstCargoShip,   
			UnitIndexFunction = nil, 
			MinTurn =  1, 
			TurnsCheck = 1, 
			TutorialLevel = 3,
			TurnTimeCheck = 2,
			Concept1 = "CONCEPT_TRADE_ROUTES",
			Concept2 = "CONCEPT_TRADE_CARAVAN",
			Concept3 = "CONCEPT_TRADE_CARGO",
		});
	end

	table.insert(TutorialInfo, { 
		ID = "STARVATION",		
		Advisor = ADVISOR_ECONOMIC,
		NotificationType = NotificationTypes.NOTIFICATION_STARVING,   
		TutorialLevel = 2,
	});

	local diploUnit = GameInfo.Units["UNIT_DIPLOMAT"];
	if(diploUnit ~= nil) then	
		table.insert(TutorialInfo, { 
			ID = "DIPLOMAT",		
			Advisor = ADVISOR_FOREIGN,
			TutorialLevel = 3,
			CheckFunction = CheckHasTech(diploUnit.PrereqTech),
			TurnTimeCheck = 2,
			Concept1 = "CONCEPT_CITY_STATE_INFLUENCE",
			Concept2 = "CONCEPT_CITY_STATE_INFLUENCE_LEVELS",
			Concept3 = "CONCEPT_CITY_STATE_DIPLOMACY_VICTORY",
		});	
	end

	local musicUnit = GameInfo.Units["UNIT_MUSICIAN"];
	if(musicUnit ~= nil) then	
		table.insert(TutorialInfo, { 
			ID = "MUSICIAN",		
			Advisor = ADVISOR_FOREIGN,
			TutorialLevel = 3,
			CheckFunction = CheckHasTech(musicUnit.PrereqTech),
			TurnTimeCheck = 2,
			Concept1 = "CONCEPT_CULTURE_TOURISM",
			Concept2 = "CONCEPT_CULTURE_TOURISM_AND_CULTURE",
			Concept3 = "CONCEPT_CULTURE_TOURISM_BENEFITS",
		});	
	end

	table.insert(TutorialInfo, { 
		ID = "MISSIONARY",		
		Advisor = ADVISOR_FOREIGN,
		TutorialLevel = 3,
		CheckFunction = FaithReligionAnd400PointCheck,
		TurnTimeCheck = 2,
		Concept1 = "CONCEPT_RELIGION_UNITS",
		Concept2 = "CONCEPT_RELIGION_SPREADING_RELIGION",
	});	

	table.insert(TutorialInfo, { 
		ID = "INQUISITOR",		
		Advisor = ADVISOR_FOREIGN,
		TutorialLevel = 3,
		CheckFunction = FaithReligionAnd200PointCheck,
		TurnTimeCheck = 2,
		Concept1 = "CONCEPT_RELIGION_UNITS",
		Concept2 = "CONCEPT_RELIGION_DEFENDING",
	});	
end