-------------------------------------------------
-- Help text for techs
-------------------------------------------------

local help_validUnitBuilds = nil;
local help_validBuildingBuilds = nil;
local help_validImprovementBuilds = nil;

function GetHelpTextForTech( iTechID )
	local pTechInfo = GameInfo.Technologies[iTechID];
	
	local pActiveTeam = Teams[Game.GetActiveTeam()];
	local pActivePlayer = Players[Game.GetActivePlayer()];
	local pTeamTechs = pActiveTeam:GetTeamTechs();
	local iTechCost = pActivePlayer:GetResearchCost(iTechID);

	local civType = GameInfo.Civilizations[pActivePlayer:GetCivilizationType()].Type;
	
	local strHelpText = "";

	-- Name
	strHelpText = strHelpText .. Locale.ToUpper(Locale.ConvertTextKey( pTechInfo.Description ));

	-- Do we have this tech?
	if (pTeamTechs:HasTech(iTechID)) then
		strHelpText = strHelpText .. " [COLOR_POSITIVE_TEXT](" .. Locale.ConvertTextKey("TXT_KEY_RESEARCHED") .. ")[ENDCOLOR]";
	end

	-- Cost/Progress
	strHelpText = strHelpText .. "[NEWLINE]-------------------------[NEWLINE]";
	
	local iProgress = pActivePlayer:GetResearchProgress(iTechID);
	local bShowProgress = true;
	
	-- Don't show progres if we have 0 or we're done with the tech
	if (iProgress == 0 or pTeamTechs:HasTech(iTechID)) then
		bShowProgress = false;
	end
	
	if (bShowProgress) then
		strHelpText = strHelpText .. Locale.ConvertTextKey("TXT_KEY_TECH_HELP_COST_WITH_PROGRESS", iProgress, iTechCost);
	else
		strHelpText = strHelpText .. Locale.ConvertTextKey("TXT_KEY_TECH_HELP_COST", iTechCost);
	end
	
	-- Leads to...
	local strLeadsTo = "";
	local bFirstLeadsTo = true;
	for row in GameInfo.Technology_PrereqTechs() do
		local pPrereqTech = GameInfo.Technologies[row.PrereqTech];
		local pLeadsToTech = GameInfo.Technologies[row.TechType];
		
		if (pPrereqTech and pLeadsToTech) then
			if (pTechInfo.ID == pPrereqTech.ID) then
				
				-- If this isn't the first leads-to tech, then add a comma to separate
				if (bFirstLeadsTo) then
					bFirstLeadsTo = false;
				else
					strLeadsTo = strLeadsTo .. ", ";
				end
				
				strLeadsTo = strLeadsTo .. "[COLOR_POSITIVE_TEXT]" .. Locale.ConvertTextKey( pLeadsToTech.Description ) .. "[ENDCOLOR]";
			end
		end
	end
	
	if (strLeadsTo ~= "") then
		strHelpText = strHelpText .. "[NEWLINE]";
		strHelpText = strHelpText .. Locale.ConvertTextKey("TXT_KEY_TECH_HELP_LEADS_TO", strLeadsTo);
	end

	-- Pre-written help text
	strHelpText = strHelpText .. "[NEWLINE]-------------------------[NEWLINE]";
	strHelpText = strHelpText .. Locale.ConvertTextKey( pTechInfo.Help );

	HelpGatherInfoAboutUniqueStuff( civType );
	strHelpText = strHelpText .. AutomaticHelpText(pTechInfo);
	
	return strHelpText;
end

function HelpGatherInfoAboutUniqueStuff( civType )

	help_validUnitBuilds = {};
	help_validBuildingBuilds = {};
	help_validImprovementBuilds = {};

	-- put in the default units for any civ
	for thisUnitClass in GameInfo.UnitClasses() do
		help_validUnitBuilds[thisUnitClass.Type]	= thisUnitClass.DefaultUnit;	
	end

	-- put in my overrides
	for thisOverride in GameInfo.Civilization_UnitClassOverrides() do
 		if thisOverride.CivilizationType == civType then
			help_validUnitBuilds[thisOverride.UnitClassType]	= thisOverride.UnitType;
 		end
	end

	-- put in the default buildings for any civ
	for thisBuildingClass in GameInfo.BuildingClasses() do
		help_validBuildingBuilds[thisBuildingClass.Type]	= thisBuildingClass.DefaultBuilding;	
	end

	-- put in my overrides
	for thisOverride in GameInfo.Civilization_BuildingClassOverrides() do
 		if thisOverride.CivilizationType == civType then
			help_validBuildingBuilds[thisOverride.BuildingClassType]	= thisOverride.BuildingType;	
 		end
	end
	
	-- add in support for unique improvements
	for thisImprovement in GameInfo.Improvements() do
		if thisImprovement.CivilizationType == civType or thisImprovement.CivilizationType == nil then
			help_validImprovementBuilds[thisImprovement.Type] = thisImprovement.Type;	
		else
			help_validImprovementBuilds[thisImprovement.Type] = nil;	
		end
	end
	
end

function AutomaticHelpText( tech )
	-- This assumes that GatherInfoAboutUniqueStuff() has been called before this
	local helpText = "";
	if tech == nil then
		return helpText;
	end

	local techType = tech.Type;

	-- add the stuff granted by this tech here
  	for thisUnitInfo in GameInfo.Units(string.format("PreReqTech = '%s'", techType)) do
 		-- if this tech grants this player the ability to make this unit
		if help_validUnitBuilds[thisUnitInfo.Class] == thisUnitInfo.Type then
			helpText = helpText .. "[NEWLINE]";
			local bIncludeRequirementsInfo = false;
			helpText = helpText .. Locale.ConvertTextKey("TXT_KEY_UNLOCKS_UNIT", thisUnitInfo.Description);
		end
 	end
 	
 	for thisBuildingInfo in GameInfo.Buildings(string.format("PreReqTech = '%s'", techType)) do
 		-- if this tech grants this player the ability to construct this building
		if help_validBuildingBuilds[thisBuildingInfo.BuildingClass] == thisBuildingInfo.Type then
			helpText = helpText .. "[NEWLINE]";
			local bExcludeName = false;
			local bExcludeHeader = false;
			helpText = helpText .. GetHelpTextForBuilding(thisBuildingInfo.ID, bExcludeName, bExcludeHeader, false, nil);
		end
 	end

 	for thisResourceInfo in GameInfo.Resources(string.format("TechReveal = '%s'", techType)) do
 		-- if this tech grants this player the ability to reveal this resource
		helpText = helpText .. "[NEWLINE]";
		helpText = helpText .. Locale.ConvertTextKey("TXT_KEY_REVEALS_RESOURCE_ON_MAP", thisResourceInfo.Description); 
 	end
 
 	for thisProjectInfo in GameInfo.Projects(string.format("TechPrereq = '%s'", techType)) do
 		-- if this tech grants this player the ability to build this project
		helpText = helpText .. "[NEWLINE]";
		local bIncludeRequirementsInfo = true;
		helpText = helpText .. GetHelpTextForProject(thisProjectInfo.ID, bIncludeRequirementsInfo);
	end

	-- if this tech grants this player the ability to perform this action (usually only workers can do these)
	for thisBuildInfo in GameInfo.Builds{PrereqTech = techType, ShowInTechTree  = 1} do
		if thisBuildInfo.ImprovementType then
			if help_validImprovementBuilds[thisBuildInfo.ImprovementType] == thisBuildInfo.ImprovementType then
				helpText = helpText .. "[NEWLINE]";
				helpText = helpText .. Locale.ConvertTextKey( thisBuildInfo.Description );
 			end
		else
			helpText = helpText .. "[NEWLINE]";
			helpText = helpText .. Locale.ConvertTextKey( thisBuildInfo.Description );
		end
	end
	
	-- show processes
	local processCondition = "TechPrereq = '" .. techType .. "'";
	for row in GameInfo.Processes(processCondition) do
		local strPText = Locale.ConvertTextKey( row.Description );
		helpText = helpText .. "[NEWLINE]";
		helpText = helpText .. Locale.ConvertTextKey( "TXT_KEY_ENABLE_PRODUCITON_CONVERSION", strPText);
	end	
		
 	-- todo: need to add abilities, etc.
	local condition = "TechType = '" .. techType .. "'";
		
	for row in GameInfo.Route_TechMovementChanges(condition) do
		helpText = helpText .. "[NEWLINE]";
		helpText = helpText .. Locale.ConvertTextKey("TXT_KEY_FASTER_MOVEMENT", GameInfo.Routes[row.RouteType].Description );
	end

	for row in GameInfo.Technology_YieldModifier(condition) do
		local yieldType = row.YieldType;
		local yieldMod = row.YieldMod;
		local yield = GameInfo.Yields[row.YieldType];
		helpText = helpText .. "[NEWLINE]";
		helpText = helpText .. Locale.ConvertTextKey("TXT_KEY_TECH_YIELD_IMPROVED", yield.Description, yieldMod);
	end
	
	-- Some improvements can have multiple yield changes, group them and THEN add text.
	local yieldChanges = {};
	for row in GameInfo.Improvement_TechYieldChanges(condition) do
		local improvementType = row.ImprovementType;
		
		if(yieldChanges[improvementType] == nil) then
			yieldChanges[improvementType] = {};
		end
		
		local improvement = GameInfo.Improvements[row.ImprovementType];
		local yield = GameInfo.Yields[row.YieldType];
		
		table.insert(yieldChanges[improvementType], Locale.Lookup( "TXT_KEY_YIELD_IMPROVED", improvement.Description , yield.Description, row.Yield));
	end
	
	-- Let's sort the yield change butons!
	local sortedYieldChanges = {};
	for k,v in pairs(yieldChanges) do
		table.insert(sortedYieldChanges, {k,v});
	end
	table.sort(sortedYieldChanges, function(a,b) return Locale.Compare(a[1], b[1]) == -1 end); 
	
	for i,v in pairs(sortedYieldChanges) do
		table.sort(v[2], function(a,b) return Locale.Compare(a,b) == -1 end);
		helpText = helpText .. "[NEWLINE]";
		helpText = helpText .. table.concat(v[2], "[NEWLINE]");
	end	
	
	for row in GameInfo.Improvement_TechNoFreshWaterYieldChanges(condition) do
		helpText = helpText .. "[NEWLINE]";
		helpText = helpText .. Locale.ConvertTextKey("TXT_KEY_NO_FRESH_WATER", GameInfo.Improvements[row.ImprovementType].Description , GameInfo.Yields[row.YieldType].Description, row.Yield);
	end	

	for row in GameInfo.Improvement_TechFreshWaterYieldChanges(condition) do
		helpText = helpText .. "[NEWLINE]";
		helpText = helpText .. Locale.ConvertTextKey("TXT_KEY_FRESH_WATER", GameInfo.Improvements[row.ImprovementType].Description , GameInfo.Yields[row.YieldType].Description, row.Yield);
	end	

	if tech.EmbarkedMoveChange > 0 then
		helpText = helpText .. "[NEWLINE]";
		helpText = helpText .. Locale.ConvertTextKey( "TXT_KEY_FASTER_EMBARKED_MOVEMENT" );
	end
	
	if tech.AllowsEmbarking then
		helpText = helpText .. "[NEWLINE]";
		helpText = helpText .. Locale.ConvertTextKey( "TXT_KEY_ALLOWS_EMBARKING" );
	end
	
	if tech.AllowsDefensiveEmbarking then
		helpText = helpText .. "[NEWLINE]";
		helpText = helpText .. Locale.ConvertTextKey( "TXT_KEY_ABLTY_DEFENSIVE_EMBARK_STRING" );
	end
	
	if tech.EmbarkedAllWaterPassage then
		helpText = helpText .. "[NEWLINE]";
		helpText = helpText .. Locale.ConvertTextKey( "TXT_KEY_ALLOWS_CROSSING_OCEANS" );
	end
	
	if tech.UnitFortificationModifier > 0 then
		helpText = helpText .. "[NEWLINE]";
		helpText = helpText .. Locale.ConvertTextKey( "TXT_KEY_UNIT_FORTIFICATION_MOD", tech.UnitFortificationModifier );
	end
	
	if tech.UnitBaseHealModifier > 0 then
		helpText = helpText .. "[NEWLINE]";
		helpText = helpText .. Locale.ConvertTextKey( "TXT_KEY_UNIT_BASE_HEAL_MOD", tech.UnitBaseHealModifier );
	end
	
	if tech.AllowEmbassyTradingAllowed then
		helpText = helpText .. "[NEWLINE]";
		helpText = helpText .. Locale.ConvertTextKey( "TXT_KEY_ALLOWS_EMBASSY" );
	end
	
	if tech.OpenBordersTradingAllowed then
		helpText = helpText .. "[NEWLINE]";
		helpText = helpText .. Locale.ConvertTextKey( "TXT_KEY_ALLOWS_OPEN_BORDERS" );
	end
	
	if tech.DefensivePactTradingAllowed then
		helpText = helpText .. "[NEWLINE]";
		helpText = helpText .. Locale.ConvertTextKey( "TXT_KEY_ALLOWS_DEFENSIVE_PACTS" );
	end
	
	if tech.ResearchAgreementTradingAllowed then
		helpText = helpText .. "[NEWLINE]";
		helpText = helpText .. Locale.ConvertTextKey( "TXT_KEY_ALLOWS_RESEARCH_AGREEMENTS" );
	end
	
	if tech.TradeAgreementTradingAllowed then
		helpText = helpText .. "[NEWLINE]";
		helpText = helpText .. Locale.ConvertTextKey( "TXT_KEY_ALLOWS_TRADE_AGREEMENTS" );
	end
	
	if tech.BridgeBuilding then
		helpText = helpText .. "[NEWLINE]";
		helpText = helpText .. Locale.ConvertTextKey( "TXT_KEY_ALLOWS_BRIDGES" );
	end

	if tech.MapVisible then
		helpText = helpText .. "[NEWLINE]";
		helpText = helpText .. Locale.ConvertTextKey( "TXT_KEY_REVEALS_ENTIRE_MAP" );
	end
	
	if tech.InternationalTradeRoutesChange > 0 then
		helpText = helpText .. "[NEWLINE]";
		helpText = helpText .. Locale.ConvertTextKey( "TXT_KEY_ADDITIONAL_INTERNATIONAL_TRADE_ROUTE" );
	end
	
	
	if (tech.ScenarioTechButton == 1) then
		helpText = helpText .. "[NEWLINE]";
		helpText = helpText .. Locale.ConvertTextKey( "TXT_KEY_SCENARIO_TECH_BUTTON_1" );
	end
	
	if (tech.ScenarioTechButton == 2) then
		helpText = helpText .. "[NEWLINE]";
		helpText = helpText .. Locale.ConvertTextKey( "TXT_KEY_SCENARIO_TECH_BUTTON_2" );
	end


	if (tech.ScenarioTechButton == 3) then
		helpText = helpText .. "[NEWLINE]";
		helpText = helpText .. Locale.ConvertTextKey( "TXT_KEY_SCENARIO_TECH_BUTTON_3" );
	end
	
	if (tech.ScenarioTechButton == 4) then
		helpText = helpText .. "[NEWLINE]";
		helpText = helpText .. Locale.ConvertTextKey( "TXT_KEY_SCENARIO_TECH_BUTTON_4" );
	end

	if (tech.ScenarioTechButton == 3) then
		helpText = helpText .. "[NEWLINE]";
		helpText = helpText .. Locale.ConvertTextKey( "Allows infantry to pillage" );
	end

	if (tech.ScenarioTechButton == 4) then
		helpText = helpText .. "[NEWLINE]";
		helpText = helpText .. Locale.ConvertTextKey( "Allows infantry to pillage" );
	end

	for row in GameInfo.Technology_TradeRouteDomainExtraRange(condition) do
		if (row.TechType == techType and row.Range > 0) then
			if (GameInfo.Domains[row.DomainType].ID == DomainTypes.DOMAIN_LAND) then
				helpText = helpText .. "[NEWLINE]";
				helpText = helpText .. Locale.ConvertTextKey( "TXT_KEY_EXTENDS_LAND_TRADE_ROUTE_RANGE" );
			elseif (GameInfo.Domains[row.DomainType].ID == DomainTypes.DOMAIN_SEA) then
				helpText = helpText .. "[NEWLINE]";
				helpText = helpText .. Locale.ConvertTextKey( "TXT_KEY_EXTENDS_SEA_TRADE_ROUTE_RANGE" );
			end	
		end
	end
	
	if tech.InfluenceSpreadModifier > 0 then
		helpText = helpText .. "[NEWLINE]";
		helpText = helpText .. Locale.ConvertTextKey( "TXT_KEY_TECH_INCREASE_TOURISM", tech.InfluenceSpreadModifier );
	end
	
	if tech.AllowsWorldCongress then
		helpText = helpText .. "[NEWLINE]";
		helpText = helpText .. Locale.ConvertTextKey( "TXT_KEY_ALLOWS_WORLD_CONGRESS" );
	end
	
	if tech.ExtraVotesPerDiplomat > 0 then
		helpText = helpText .. "[NEWLINE]";
		helpText = helpText .. Locale.ConvertTextKey( "TXT_KEY_EXTRA_VOTES_FROM_DIPLOMATS", tech.ExtraVotesPerDiplomat );
	end

	if tech.StealTechFasterModifier > 0 then
		helpText = helpText .. "[NEWLINE]";
		helpText = helpText .. Locale.ConvertTextKey( "TXT_KEY_TECH_STEAL_TECH_MODIFIER", tech.StealTechFasterModifier );
	end

	if tech.RiggingElectionModifier > 0 then
		helpText = helpText .. "[NEWLINE]";
		helpText = helpText .. Locale.ConvertTextKey( "TXT_KEY_TECH_RIGGING_ELECTION_MODIFIER", tech.RiggingElectionModifier );
	end

	if tech.CatchSpiesModifier > 0 then
		helpText = helpText .. "[NEWLINE]";
		helpText = helpText .. Locale.ConvertTextKey( "TXT_KEY_TECH_CATCH_SPIES_MODIFIER", tech.CatchSpiesModifier );
	end

	if not(tech.EnemyStealTechSlowerModifier == 0) then
		helpText = helpText .. "[NEWLINE]";
		helpText = helpText .. Locale.ConvertTextKey( "TXT_KEY_TECH_ENEMY_STEAL_TECH_MODIFIER", -tech.EnemyStealTechSlowerModifier );
	end
	
	for row in GameInfo.Technology_FreePromotions(condition) do
		local promotion = GameInfo.UnitPromotions[row.PromotionType];
		if promotion ~= nil then
			helpText = helpText .. "[NEWLINE]";
			helpText = helpText .. Locale.ConvertTextKey("TXT_KEY_FREE_PROMOTION_FROM_TECH", promotion.Description, promotion.Help);
		else
			break;
		end
	end
	
	return helpText;
	
end